#!/usr/bin/env python3
# -*- coding: future_fstrings -*-

"""
This module is used to check for myisam tables
"""
from migrationsummary.parsing.logging.LogParser import LogParser
from migrationsummary.types.SQL import SQL


def check_myisam() -> set:
    """
    Retrieves a list of all myisam databases
    """

    sql_query = """SELECT
      `table_schema` AS 'database_name',
      `TABLE_NAME` AS 'table_name'
    FROM
      `information_schema`.`tables`
    WHERE ENGINE = 'MyISAM' AND
      `table_type` = 'BASE TABLE' AND
      `table_schema` NOT IN(
        'information_schema',
        'sys',
        'performance_schema',
        'mysql'
      )
    ORDER BY
      `table_schema`,
      `TABLE_NAME`;"""

    db_ispc = SQL.db_query(sql_stmt=sql_query)
    return db_ispc.result


def summary() -> str:
    log_parser = LogParser()
    level = log_parser.set_log_level(log_parser.OK)
    db_myisam = check_myisam()

    sum_str = ''

    if db_myisam:
        level = log_parser.set_log_level(log_parser.WARN)
        outfile = '/root/migrationsummary-myisam.log'
        sum_str = (f"[{level}] Found {len(db_myisam)} myisam tables please check "
                      f"{outfile} for more details\n")
        with open(f'{outfile}', 'w') as myisamlog:
            for item in db_myisam:
                myisamlog.write(f"{item['database_name']}.{item['table_name']}\n")

    sum_str = (f'[{level}] No issues found\n') if sum_str == '' else sum_str

    sub_mod = '--> \033[46mSubmodule: MyISAM\033[0m\n'
    sum_str = sub_mod + sum_str

    return sum_str


if __name__ == '__main__':
    print(summary())
