#!/usr/bin/env python3
# -*- coding: future_fstrings -*-

"""
This module is used to check for network related settings like additional
ipv4 addresses and print a warning to let the user know these addresses need
to be mapped to new addresses or onto a single address
"""
from migrationsummary.ispconfig import ISPConfig
from migrationsummary.parsing.logging.LogParser import LogParser
from migrationsummary.types import SQL
from os import popen
from types import GeneratorType


def get_ip_mapping() -> dict:
    command = ("/usr/local/bin/ispcphp"
               " /usr/local/ispconfig/server/scripts/console/ISPCConsole.php"
               " --show-vhosts --raw --select=ip_address"
               " | awk -F, '{print $5,$NF}'")
    results = popen(command).readlines()
    domains = map(lambda d: d.strip().split(), results[0:])
    ipdict = dict()
    for domain in domains:
        try:
            if domain[1] in ipdict:
                ipdict[domain[1]].append(domain[0])
            elif domain[1] != '':
                ipdict[domain[1]] = [domain[0]]
        except IndexError:
            # ignore empty entries for now
            pass

    return ipdict


def get_ispconfig_ip_addresses() -> GeneratorType:
    sql_stmt = (f"SELECT `ip_address` FROM `server_ip`"
                f" WHERE `server_id` = {ISPConfig.SERVER_ID}")
    for res in SQL.db_query(sql_stmt).result:
        if 'error' in res:
            yield ''
        if 'ip_address' in res:
            yield res['ip_address']


def summary() -> str:
    ips = [ip for ip in get_ispconfig_ip_addresses() if ip != '']
    log_parser = LogParser()
    if len(ips) > 1:
        ipdict = get_ip_mapping()
        level = log_parser.set_log_level(log_parser.WARN)
        sum_str = f'[{level}] multiple IP Addresses found:\n'

        for ip in ips:
            sum_str += f"    \\_ {ip}\n"
            try:
                for domain in ipdict[ip]:
                    sum_str += f"        \\_ {domain}\n"
            except KeyError:
                pass
    else:
        level = log_parser.set_log_level(log_parser.OK)
        sum_str = f'[{level}] only one IP Address found'

    return sum_str


if __name__ == '__main__':
    print(summary())
