#!/usr/bin/env python3
# -*- coding: future_fstrings -*-
from migrationsummary.parsing.logging.LogParser import LogParser
from migrationsummary.models.Summary import Summary
from migrationsummary.parsing.template.TemplateRenderer import TemplateRenderer
import os
import csv


class MailDomains(Summary):

    def __init__(self):
        super().__init__(plugin_name='maildomains',
                         template='maildomains_summary.j2')

    def checkMailDomains(self):
        command = ('/usr/local/ispconfig/server/scripts/console/ISPCConsole.php'
                   ' --raw --show-mail-domains')
        stream = os.popen(command)
        maildomains_list = stream.read()
        stream.close()
        return maildomains_list

    def mx(self, domain):
        command = f"dig mx +short {domain} | cut -d' ' -f 2"
        stream = os.popen(command)
        mx_entry = stream.read().strip()
        stream.close()
        return mx_entry

    def summarize(self) -> None:
        log_parser = LogParser()
        tpl_init = TemplateRenderer(self.plugin_name, self.template)
        tpl_str = tpl_init.template_render()
        # Print module header
        print(tpl_str.strip())
        output = self.checkMailDomains()
        output = output.strip()
        lines = output.splitlines()
        reader = csv.reader(lines)
        output = list(reader)
        domain_count = len(output)
        if domain_count > 0:
            log_level = log_parser.set_log_level(log_parser.OK)
            sum_str = f"[{log_level}] Detected {domain_count} Mail Domains\n"
            for domain_entry in output:
                sum_str += (f"   \\_ Domain: {domain_entry[2]}, "
                            f"DKIM active: {domain_entry[3]}\n"
                            f"      MX Entry for Domain: "
                            f"{self.mx(domain_entry[2])}\n")
        else:
            log_level = log_parser.set_log_level(log_parser.OK)
            sum_str = f"[{log_level}] No Mail Domains\n"

        print(sum_str)


if __name__ == '__main__':
    None
