#!/usr/bin/env python3
# -*- coding: future_fstrings -*-

from abc import ABC
from collections import defaultdict
import migrationsummary


class Summary(ABC):
    ("""Structure
    Properties:
    - plugin_name: str
    - summary: defaultdict
    - summeries: list of dicts""")

    def __init__(self, plugin_name: str, template: str):
        self._plugin_name = plugin_name
        self._template = template
        self._summary = defaultdict()

    @property
    def plugin_name(self) -> str:
        return self._plugin_name

    @plugin_name.setter
    def plugin_name(self, plugin_name: str) -> None:
        self._plugin_name = plugin_name

    @property
    def template(self) -> str:
        return self._template

    @template.setter
    def template(self, tpl) -> None:
        self._template = tpl

    @property
    def summary(self) -> defaultdict:
        return self._summary

    @summary.setter
    def summary(self, summary: defaultdict) -> None:
        if type(summary) != defaultdict:
            raise TypeError
        self._summary = summary

    @property
    def plugin_line(self) -> str:
        "readonly"
        return (f'-------- {self._plugin_name} -------------------------------'
                '-----------------------------------')

    @property
    def fork(self) -> str:
        "readonly"
        return '\\_'

    @property
    def context_line(self) -> str:
        return self._context_line

    @context_line.setter
    def context_line(self, value) -> None:
        self._context_line = value

    @property
    def module_path(self) -> str:
        return ''.join(migrationsummary.__path__)

    def summarize(self) -> None:
        pass
