#!/usr/bin/env python3
# -*- coding: future_fstrings -*-
from typing import Any, Generator
from yaml import safe_load
from yaml.error import YAMLError


class YamlFile:
    def __init__(self, file_path, file_op):
        self._yaml_file = file_path
        self._file_op = file_op

    @property
    def yaml_file(self):
        return self._yaml_file

    @yaml_file.setter
    def yaml_file(self, file_path):
        self._yaml_file = file_path

    @property
    def file_op(self):
        return self._file_op

    @file_op.setter
    def file_op(self, file_op):
        self._file_op = file_op

    def read(self) -> Generator[Any, Any, Any]:
        "Read target yaml file line by line"
        try:
            with open(self._yaml_file, self._file_op) as stream:
                yield stream.read()
        except FileNotFoundError as e:
            yield Exception(f'{e}')


class YamlParser:
    def __init__(self, file_path, file_op):
        self._yaml_file = YamlFile(file_path=file_path, file_op=file_op)

    @property
    def yaml_file(self) -> YamlFile:
        return self._yaml_file

    @yaml_file.setter
    def yaml_file(self, yaml_file) -> None:
        self._yaml_file = yaml_file

    def parse(self, data: str) -> Any:
        "Parse yaml content from target"
        yaml_data = set()
        try:
            yaml_data = safe_load(data)
        except YAMLError as e:
            yaml_data = e

        return yaml_data
