# -*- coding: utf-8 -*-
from collections import defaultdict
# Python 3.10 removed collections.Iterable
try:
    from collections import Iterable
except ImportError:
    from collections.abc import Iterable

from migrationsummary.parsing.logging.LogParser import LogParser
from migrationsummary.models.Summary import Summary
from migrationsummary.parsing.template.TemplateRenderer import TemplateRenderer
from os import path
from posix import listdir
from typing import Generator


class custom_enumerate(enumerate):

    def __init__(self, iter: Iterable):
        self.iter = iter
        super().__init__()

    @property
    def iter(self) -> Iterable:
        return self._iter

    @iter.setter
    def iter(self, iter: Iterable) -> None:
        self._iter = iter

    @property
    def length(self):
        return sum(1 for _ in self.iter)

    def __iter__(self) -> Generator:
        for idx, item in enumerate(self.iter):
            idx_next = idx + 1
            if idx_next == self.length:
                break
            yield idx, idx_next, item


class PHPDevSock(Summary):
    def __init__(self):
        super().__init__(
            plugin_name='PHP Development Sockets',
            template='php_dev_sockets_summary.j2'
        )

    @property
    def path_sites_enabled(self) -> str:
        return '/etc/nginx/sites-enabled'

    def get_dev_sockets(self) -> Generator:
        log_parser = LogParser()
        # php_dev_socks = defaultdict(None)
        sites_enabled = listdir(self.path_sites_enabled)
        list_config_skip = [
            '000-apps.vhost',
            '000-ispconfig.vhost',
            'default',
            '000-default'
        ]
        for conf in sites_enabled:
            if conf in list_config_skip:
                continue
            abs_path_conf = path.join(self.path_sites_enabled, conf)
            self.summary = defaultdict(None)
            with open(abs_path_conf, 'rb') as stream:
                to_list = stream\
                    .read()\
                    .decode(encoding='utf-8')\
                    .split()
            stream.close()
            fpm_sockets = []
            for idx, idx_next, item in custom_enumerate(to_list):
                if item == 'fastcgi_pass':
                    fpm_sockets.append(to_list[idx_next].strip(';'))

            if fpm_sockets:
                diff = []
                for idx, idx_next, item in custom_enumerate(fpm_sockets):
                    if item != fpm_sockets[idx_next]:
                        diff.append(fpm_sockets[idx_next])
                if diff:
                    self.summary['config'] = abs_path_conf
                    self.summary['sockets'] = diff
                    self.summary['level'] = log_parser\
                        .set_log_level(log_parser.WARN)
                    yield self.summary
        return None

    def no_dev_sockets_found(self):
        log_parser = LogParser()
        return defaultdict(None, {
            'level': log_parser.set_log_level(log_parser.OK),
            'message': 'No development sockets found!'
        })

    def summarize(self) -> None:
        list_dev_socks = []
        if any(self.get_dev_sockets()):
            for dict_item in self.get_dev_sockets():
                if dict_item:
                    list_dev_socks.append(dict_item)
        tpl_init = TemplateRenderer(plugin_name=self.plugin_name,
                                    tpl_name=self.template)
        tpl_str = tpl_init.template_render(**{
            'dev_socks': (list_dev_socks
                          if list_dev_socks
                          else self.no_dev_sockets_found())
        })
        print(tpl_str)
