#!/usr/bin/env python3
# -*- coding: future_fstrings -*-
from collections import defaultdict
import migrationsummary.funcs as func
from os.path import isfile
import pymysql as my
from pymysql.cursors import DictCursor
from pymysql.connections import Connection


class SQL(object):

    def __init__(self, query: str, result: defaultdict) -> None:
        self.query = query
        self.result = result

    @property
    def query(self) -> str:
        return self._query

    @query.setter
    def query(self, query: str) -> None:
        self._query = query

    @property
    def result(self) -> defaultdict:
        return self._result

    @result.setter
    def result(self, result: defaultdict) -> None:
        self._result = result

    @classmethod
    def db_conf(cls, use_db: str, useunicode: bool = True) -> Connection:
        """
        Parse MySQL Configuration of ISPConfig and return object of type
        Connection
        :param use_db:
        :param useunicode:
        :return:
        """
        sql_conf = '/usr/local/ispconfig/server/lib/mysql_clientdb.conf'
        if not isfile(sql_conf):
            raise Exception(f'Fatal: {sql_conf} - No such file or directory')
        try:
            config = []
            with open(sql_conf, 'r') as ro_stream:
                line = ro_stream.readline()
                while line != '':
                    # Skip php tags
                    if (line.startswith('<?php')
                            or line.startswith('?>')):
                        line = ''

                    # Strip "$" from line
                    if line.startswith('$'):
                        line = line[1:]

                    # Trim unnecessary content
                    line = func.trim(line)

                    # Strip ";" from line
                    if line and line.endswith(';'):
                        line = line[:len(line) - 1]

                    # Append to config
                    if line:
                        config.append(line.split('='))
                    # Read next line
                    line = ro_stream.readline()
            ro_stream.close()

            # Create config dictionary
            config_dict = {}
            if config:
                for arr in config:
                    # Dict Comprehension to create key/value pair from arr
                    part_dict = {arr[i]: arr[i + 1]
                                 for i in range(0, len(arr), 2)}
                    # Merge part_dict into config_dict
                    config_dict = {**config_dict, **part_dict}
            return my.connect(host=config_dict['clientdb_host'],
                              user=config_dict['clientdb_user'],
                              password=config_dict['clientdb_password'],
                              use_unicode=useunicode,
                              cursorclass=DictCursor,
                              database=use_db)
        except my.err.OperationalError:
            raise Exception('Fatal: Unable to connect to database server.')

    @classmethod
    def db_query(cls,
                 sql_stmt: str,
                 db: str = 'dbispconfig',
                 fetch_all: bool = True,
                 count: int = 0):
        try:
            sql_conn = cls.db_conf(use_db=db)
            try:
                sql_cursor = sql_conn.cursor()
                result = defaultdict(None)
                if not sql_cursor.execute(query=sql_stmt):
                    return cls(query=sql_stmt, result=result)
                if not fetch_all:
                    result = sql_cursor.fetchone()
                if fetch_all and count > 0:
                    result = sql_cursor.fetchmany(count)
                if fetch_all and count == 0:
                    result = sql_cursor.fetchall()
            except Exception as e:
                print('Fatal: could not run sql query.')
                result = defaultdict(None, {'error': e})
            sql_conn.close()
        except Exception as e:
            print('Fatal: Unable to open connection to database server.')
            result = defaultdict(None, {'error': e})
        return cls(query=sql_stmt, result=result)
